/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Rick Potts <rpotts@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "nsISupports.idl"


[uuid(b6427ace-dc77-495b-ab21-43baeb52bc27)]
interface nsIScriptEventManager : nsISupports
{
 /*
  * FindEventHandler:
  *
  * Returns a script event handler for the given object and event name.
  *
  * @param aObjectName    Name of the object raising the event.
  *
  * @param aEventName     Name of the event being raised.
  *
  * @param aArgCount      Number of arguments expected by the event handler.
  *
  */
  nsISupports FindEventHandler(in AString aObjectName,
                               in AString aEventName,
                               in unsigned long aArgCount);

 /*
  *
  * InvokeEventHandler:
  *
  * Executes the script handler that was returned from FindEventHandler.
  *
  * @param aHandler       Script event handler that was returned from
  *                       FindEventHandler.
  *
  * @param aTargetObject  Object to which the script handler is bound.
  *
  * @param aArgs          Array of arguments passed to the script handler.
  *                       This is an array of jsvals.
  *
  * @param aArgCount      Number of elements in the aArgs array.
  *
  */
  void InvokeEventHandler(in nsISupports aHandler,
                          in nsISupports aTargetObject,
                          in voidPtr aArgs,
                          in unsigned long aArgCount);
};

